@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: client_channel.c,v 1.36 2002/06/16 07:35:23 priikone Exp $ */
+/* $Id: client_channel.c,v 1.37 2002/06/19 12:04:03 priikone Exp $ */
 /* This file includes channel message sending and receiving routines,
    channel key receiving and setting, and channel private key handling 
    routines. */
@@ -302,7 +302,8 @@ void silc_client_channel_message(SilcClient client,
 
   /* Find client entry */
   client_entry = silc_client_get_client_by_id(client, conn, client_id);
-  if (!client_entry || !client_entry->nickname) {
+  if (!client_entry || !client_entry->nickname ||
+      !silc_client_on_channel(channel, client_entry)) {
     /* Resolve the client info */
     SilcChannelClientResolve res = silc_calloc(1, sizeof(*res));
     res->payload = payload;
@@ -315,11 +316,6 @@ void silc_client_channel_message(SilcClient client,
     goto out;
   }
 
-  if (!silc_client_on_channel(channel, client_entry)) {
-    SILC_LOG_WARNING(("Received channel message from client not on channel"));
-    goto out;
-  }
-
   message = silc_channel_message_get_data(payload, &message_len);
 
   /* Pass the message to application */
