@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: command_reply.c,v 1.105 2002/05/17 10:15:34 priikone Exp $ */
+/* $Id: command_reply.c,v 1.106 2002/06/22 20:53:16 priikone Exp $ */
 /*
  * Command reply functions are "the otherside" of the command functions.
  * Reply to a command sent by server is handled by these functions.
@@ -432,8 +432,12 @@ silc_client_command_reply_identify_save(SilcClientCommandReplyContext cmd,
 	  COMMAND_REPLY_ERROR;
 	return;
       }
+    } else {
+      silc_client_update_server(client, conn, server_entry, name, info);
     }
 
+    server_entry->resolve_cmd_ident = 0;
+
     /* Notify application */
     if (notify)
       COMMAND_REPLY((ARGS, server_entry, name, info));
