@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.51 2002/05/03 11:08:20 priikone Exp $ */
+/* $Id: idlist.c,v 1.54 2002/06/22 20:53:16 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcclient.h"
@@ -287,6 +287,7 @@ typedef struct {
   SilcBuffer client_id_list;
   SilcGetClientCallback completion;
   void *context;
+  int res_count;
 } *GetClientsByListInternal;
 
 SILC_CLIENT_CMD_FUNC(get_clients_list_callback)
@@ -301,6 +302,14 @@ SILC_CLIENT_CMD_FUNC(get_clients_list_callback)
 
   SILC_LOG_DEBUG(("Start"));
 
+  if (i->res_count) {
+    i->res_count--;
+    if (i->res_count)
+      return;
+  }
+
+  SILC_LOG_DEBUG(("Resolved all clients"));
+
   for (c = 0; c < i->list_count; c++) {
     SilcUInt16 idp_len;
     SilcClientID *client_id;
@@ -410,6 +419,7 @@ void silc_client_get_clients_by_list(SilcClient client,
 			    silc_client_command_get_clients_list_callback, 
 			    (void *)in);
 	  wait_res = TRUE;
+	  in->res_count++;
 
 	  silc_free(client_id);
 	  silc_buffer_pull(client_id_list, idp_len);
@@ -421,7 +431,7 @@ void silc_client_get_clients_by_list(SilcClient client,
       }
 
       /* No we don't have it, query it from the server. Assemble argument
-	 table that will be sent fr the IDENTIFY command later. */
+	 table that will be sent for the IDENTIFY command later. */
       res_argv = silc_realloc(res_argv, sizeof(*res_argv) *
 			      (res_argc + 1));
       res_argv_lens = silc_realloc(res_argv_lens, sizeof(*res_argv_lens) *
@@ -463,6 +473,7 @@ void silc_client_get_clients_by_list(SilcClient client,
     silc_client_command_pending(conn, SILC_COMMAND_IDENTIFY, conn->cmd_ident,
 				silc_client_command_get_clients_list_callback, 
 				(void *)in);
+    in->res_count++;
 
     silc_buffer_free(res_cmd);
     silc_free(res_argv);
@@ -965,6 +976,8 @@ SilcServerEntry silc_client_add_server(SilcClient client,
 {
   SilcServerEntry server_entry;
 
+  SILC_LOG_DEBUG(("Start"));
+
   server_entry = silc_calloc(1, sizeof(*server_entry));
   if (!server_entry || !server_id)
     return NULL;
@@ -1001,6 +1014,34 @@ bool silc_client_del_server(SilcClient client, SilcClientConnection conn,
   return ret;
 }
 
+/* Updates the `server_entry' with the new information sent as argument. */
+
+void silc_client_update_server(SilcClient client,
+			       SilcClientConnection conn,
+			       SilcServerEntry server_entry,
+			       const char *server_name,
+			       const char *server_info)
+{
+  SILC_LOG_DEBUG(("Start"));
+
+  if (server_name && (!server_entry->server_name ||
+		      strcmp(server_entry->server_name, server_name))) {
+
+    silc_idcache_del_by_context(conn->server_cache, server_entry);
+    silc_free(server_entry->server_name);
+    server_entry->server_name = strdup(server_name);
+    silc_idcache_add(conn->server_cache, server_entry->server_name,
+		     server_entry->server_id,
+		     server_entry, 0, NULL);
+  }
+
+  if (server_info && (!server_entry->server_info ||
+		      strcmp(server_entry->server_info, server_info))) {
+    silc_free(server_entry->server_info);
+    server_entry->server_info = strdup(server_info);
+  }
+}
+
 /* Formats the nickname of the client specified by the `client_entry'.
    If the format is specified by the application this will format the
    nickname and replace the old nickname in the client entry. If the
@@ -1069,6 +1110,9 @@ void silc_client_nickname_format(SilcClient client,
       if (!client_entry->hostname)
 	break;
       len = strcspn(client_entry->hostname, ".");
+      i = strcspn(client_entry->hostname, "-");
+      if (i < len)
+        len = i;
       newnick = silc_realloc(newnick, sizeof(*newnick) * (off + len));
       memcpy(&newnick[off], client_entry->hostname, len);
       off += len;
